'''Generator and Discriminator for GAN.'''
import torch
import torch.nn.functional as F
import torch.nn as nn

#Generator for the GAN
class Generator(nn.Module):
    '''Generator Module for GAN.'''
    def __init__(self , normalization = None):
        super(Generator , self).__init__()

        # downsampling layers
        self.d1 = ConvolutionBlock(1 , 64 ,
                                   normalization = normalization ,
                                   kernel_size = 4 ,
                                   stride = 1 ,
                                   padding = 0 ,
                                   dropout = None
                                   )
        self.d2 = ConvolutionBlock(64, 128,
                                   normalization=normalization,
                                   kernel_size=4,
                                   stride=2,
                                   padding=1,
                                   dropout=None
                                   )
        self.d3 = ConvolutionBlock(128, 256,
                                   normalization=normalization,
                                   kernel_size=4,
                                   stride=2,
                                   padding=1,
                                   dropout=None
                                   )
        self.d4 = ConvolutionBlock(256, 512,
                                   normalization=normalization,
                                   kernel_size=4,
                                   stride=2,
                                   padding=1,
                                   dropout=None
                                   )
        self.d5 = ConvolutionBlock(512, 512,
                                   normalization=normalization,
                                   kernel_size=4,
                                   stride=2,
                                   padding=1,
                                   dropout=None
                                   )
        # upsampling layers
        self.u1 = TransposeConvolutionBlock(512 ,512 ,
                                            normalization=normalization,
                                            kernel_size=4,
                                            stride=2,
                                            padding=1,
                                            dropout=None
                                            )
        self.u2 = TransposeConvolutionBlock(1024, 256,
                                            normalization=normalization,
                                            kernel_size=4,
                                            stride=2,
                                            padding=1,
                                            dropout=None
                                            )
        self.u3 = TransposeConvolutionBlock(512, 128,
                                            normalization=normalization,
                                            kernel_size=4,
                                            stride=2,
                                            padding=1,
                                            dropout=None
                                            )
        self.u4 = TransposeConvolutionBlock(256, 64,
                                            normalization=normalization,
                                            kernel_size=4,
                                            stride=2,
                                            padding=1,
                                            dropout=None
                                            )

        self.final_layer = ConvolutionBlock(128 , 2 ,
                                            normalization = None,
                                            kernel_size=1,
                                            stride=1,
                                            padding=0,
                                            dropout=None,
                                            activation_function=nn.Tanh()
                                            )

    def forward(self , x):
        '''Forward Method for Generator.'''
        x = F.interpolate(x, size=(35,35),mode='bilinear',align_corners=True)
        # print(f'Initial: {x.shape}')
        # downsampling layers
        d1 = self.d1(x)
        d2 = self.d2(d1)
        d3 = self.d3(d2)
        d4 = self.d4(d3)
        d5 = self.d5(d4)
        # print(f'Shape d1: {d1.shape}')
        # print(f'Shape d2: {d2.shape}')
        # print(f'Shape d3: {d3.shape}')
        # print(f'Shape d4: {d4.shape}')
        # print(f'Shape d5: {d5.shape}')

        # upsampling layers with U Net Structure
        u1 = self.u1(d5,d4)
        u2 = self.u2(u1,d3)
        u3 = self.u3(u2,d2)
        u4 = self.u4(u3,d1)

        # print(f'Shape u1: {u1.shape}')
        # print(f'Shape u2: {u2.shape}')
        # print(f'Shape u3: {u3.shape}')
        # print(f'Shape u4: {u4.shape}')

        final = self.final_layer(u4)
        # print(f'Final: {final.shape}')
        return final


# Discriminator Module for GAN
class Discriminator(nn.Module):
    '''Discriminator Module for GAN.'''
    def __init__(self,
                 normalization=None):
        super(Discriminator , self).__init__()

        # downsampling layers
        # similar to Donsampling in Generator
        self.d1 = ConvolutionBlock(3, 64,
                                   normalization=normalization,
                                   kernel_size=4,
                                   stride=1,
                                   padding=0,
                                   dropout=None
                                   )
        self.d2 = ConvolutionBlock(64, 128,
                                   normalization=normalization,
                                   kernel_size=4,
                                   stride=2,
                                   padding=1,
                                   dropout=None
                                   )
        self.d3 = ConvolutionBlock(128, 256,
                                   normalization=normalization,
                                   kernel_size=4,
                                   stride=2,
                                   padding=1,
                                   dropout=None
                                   )
        self.d4 = ConvolutionBlock(256, 512,
                                   normalization=normalization,
                                   kernel_size=4,
                                   stride=2,
                                   padding=1,
                                   dropout=None
                                   )

        self.final_layer = ConvolutionBlock(512,1,
                                            normalization=None,
                                            kernel_size=4,
                                            stride=1,
                                            padding=0,
                                            dropout=None,
                                            activation_function=nn.Sigmoid()
                                            )
    def forward(self,x):
        '''Forward method for Discriminator Module.'''
        x = F.interpolate(x,size=(35,35), mode='bilinear', align_corners=True)
        d1 = self.d1(x)
        d2 = self.d2(d1)
        d3 = self.d3(d2)
        d4 = self.d4(d3)

        # print(f'Shape d1: {d1.shape}')
        # print(f'Shape d2: {d2.shape}')
        # print(f'Shape d3: {d3.shape}')
        # print(f'Shape d4: {d4.shape}')

        final_out = self.final_layer(d4)
        # print(f'Final intial: {final_out.shape}')
        final_out = final_out.view(x.size()[0],-1)
        # print(f'Final final: {final_out.shape}')
        return final_out



# Convolution Layer Block for Downsampling Operation
class ConvolutionBlock(nn.Module):
    '''Class for Convolution Blocks for Downsampling.'''
    def __init__(self , input_channel , output_channel ,
                 normalization = None,
                 kernel_size  = 4,
                 stride  = 2,
                 padding  = 1,
                 dropout  = None,
                 activation_function = nn.ReLU() ):
        super(ConvolutionBlock ,  self).__init__()
        model_layers = []

        # Appending the Main Convolution Layer
        model_layers.append(
            nn.Conv2d(input_channel , output_channel ,
                      kernel_size = kernel_size ,
                      stride = stride ,
                      padding = padding)
        )

        #Applying Normalization Layer
        if normalization is not None:
            if normalization == 'Batch':
                model_layers.append(
                    nn.BatchNorm2d(output_channel)
                )
            elif normalization == 'Instance':
                model_layers.append(
                    nn.InstanceNorm2d(output_channel)
                )
        # Appending Activation Function given in the input
        model_layers.append(activation_function)

        # If Dropout is applicable
        if dropout is not None:
            model_layers.append(
                nn.Dropout(dropout)
            )
        # Making the Sequential Model
        model = nn.Sequential(*model_layers)
        self.model = model

    def forward(self , x):
        '''Forward Operation for Convolution Layer'''
        return self.model(x)

# Transpose Convolution block for Upsampling Operation
class TransposeConvolutionBlock(nn.Module):
    '''Transpose Convolution Block for Upsampling Architecture.'''
    def __init__(self, input_channel, output_channel,
                 normalization=None,
                 kernel_size=4,
                 stride=2,
                 padding=1,
                 dropout=None,
                 activation_function=nn.ReLU()):
        super(TransposeConvolutionBlock , self).__init__()
        model_layers = []

        # Adding the Transpose Convolution Layer
        model_layers.append(
            nn.ConvTranspose2d(input_channel , output_channel ,
                               kernel_size = kernel_size ,
                               stride = stride ,
                               padding = padding)
        )

        # adding normalization
        if normalization is not None:
            if normalization == 'Batch':
                model_layers.append(
                    nn.BatchNorm2d(output_channel)
                )
            elif normalization == 'Instance':
                model_layers.append(
                    nn.InstanceNorm2d(output_channel)
                )

        # adding activation function for the layer
        model_layers.append(
            activation_function
        )

        # adding dropout
        if dropout is not None:
            model_layers.append(
                nn.Dropout(dropout)
            )

        # creating the sequential model
        model = nn.Sequential(*model_layers)
        self.model = model

    def forward(self , x1 , skip_input):
        '''Forward Method for the input.'''
        x = self.model(x1)
        return torch.cat((x , skip_input) , 1)

# # Testing
# if __name__=='__main__':
#     gen = Discriminator()
#     sample_input = torch.zeros((1,1,32,32))
#     print(sample_input.shape)
#     print(gen)
#     print(gen(sample_input))










'''Data Handler Functions for Colour-GAN.'''
import os
import cv2
import pickle
import random
import tarfile
import urllib.request
import numpy as np
from torch.utils.data import DataLoader, Dataset
from colourgan.logger import log

class Cifar10(Dataset):
    def __init__(self,
                 data_dir,
                 mirror=False,
                 random_seed=None):
        self.imgs_paths = [os.path.join(data_dir,f) for f in os.listdir(data_dir)]
        if random_seed is not None:
            self.imgs_paths.sort()
            random.Random(random_seed).shuffle(self.imgs_paths)
        self.mirror = mirror

    def __len__(self):
        return len(self.imgs_paths)

    def __getitem__(self, id):
        img_path = self.imgs_paths[id]
        img_bgr = cv2.imread(img_path)

        # just a random Transformation for better training
        if self.mirror:
            if random.random() > 0.5:
                img_bgr = img_bgr[:, ::-1, :]

        img_bgr = img_bgr.astype(np.float32) / 255.0
        # transform to LAB
        img_lab = cv2.cvtColor(img_bgr, cv2.COLOR_BGR2LAB)
        img_lab[:, :, 0] = img_lab[:, :, 0] / 50 - 1
        img_lab[:, :, 1] = img_lab[:, :, 1] / 127
        img_lab[:, :, 2] = img_lab[:, :, 2] / 127
        img_lab = img_lab.transpose((2, 0, 1))

        return img_lab

class Cifar10Dataset:
    '''Class for Dwonloading and Processing Cifar 10 Dataset.'''
    def __init__(self,
                 dataset_path='cifar10',
                 batch_size=8,
                 num_workers=4):
        self.batch_size = batch_size
        self.dataset_path = dataset_path

        self.download_cifar10()
        datasets = self.process_and_split_cifar10()
        train_data = Cifar10(datasets['train'])
        test_data = Cifar10(datasets['test'])

        data_loaders = {
            'train': DataLoader(train_data,batch_size=self.batch_size, shuffle=True,
                                num_workers=num_workers),
            'test': DataLoader(test_data, batch_size=self.batch_size, shuffle=False,
                               num_workers=num_workers)
        }
        self.data_loaders = data_loaders

    def get_dataloaders(self):
        return self.data_loaders

    def download_cifar10(self):
        '''Method for Downloading CIFAR10 dataset.'''
        if not os.path.exists(self.dataset_path):
            os.makedirs(self.dataset_path)
            log(f'{self.dataset_path} Directory Created' , 'data.py/Cifar10Dataset')
            log('Downloading Cifar 10', 'data.py/Cifar10Dataset')
            urllib.request.urlretrieve('https://www.cs.toronto.edu/~kriz/cifar-10-python.tar.gz',
                                       os.path.join(self.dataset_path, 'cifar-10-python.tar.gz'))

            log('Unzipping Dataset', 'data.py/Cifar10Dataset')
            tar = tarfile.open(os.path.join(self.dataset_path, 'cifar-10-python.tar.gz'), 'r:gz')
            tar.extractall(path=self.dataset_path)
            tar.close()
        else:
            log('Downloaded Dataset Found', 'data.py/Cifar10Dataset')
        return



    def process_and_split_cifar10(self):
        '''Extracting, Processing and Splitting Cifar10 Dataset.'''
        data_batches = {}
        data_batches['train'] = [
            os.path.join(self.dataset_path,'cifar-10-batches-py',f) for f in [
                'data_batch_1' , 'data_batch_2' , 'data_batch_3' , 'data_batch_4' , 'data_batch_5'
            ]
        ]
        data_batches['test'] = [os.path.join(self.dataset_path,'cifar-10-batches-py','test_batch')]
        data_dir = {}
        # Directories for Train and Test Split
        data_dir['train'] = os.path.join(self.dataset_path,'cifar-10-images','train')
        data_dir['test'] = os.path.join(self.dataset_path, 'cifar-10-images', 'test')

        for task in ['train' , 'test']:
            if not os.path.exists(data_dir[task]):
                os.makedirs(data_dir[task])
                log(f'{data_dir[task]} Directory Created', 'data.py/Cifar10Dataset')
                log(f'Extracting {task} Dataset', 'data.py/Cifar10Dataset')
                for batch_path in data_batches[task]:
                    with open(batch_path,'rb') as f:
                        batch = pickle.load(f, encoding='bytes')
                    if task == 'train':
                        print(len(batch[b'filenames']))
                    for image_name,image_vector in zip(batch[b'filenames'],batch[b'data']):
                        r, g, b = image_vector[0:1024], image_vector[1024:2048], image_vector[2048:]
                        r, g, b = np.reshape(r, (32, -1)), np.reshape(g, (32, -1)), np.reshape(b, (32, -1))
                        img = np.stack((b, g, r), axis=2)

                        save_path = os.path.join(data_dir[task],image_name.decode('utf-8'))
                        cv2.imwrite(save_path,img)
            else:
                log(f'{task} Split Found', 'data.py/Cifar10Dataset')
        return data_dir

# if __name__ == '__main__':
#     # for testing
#     dataloaders = Cifar10Dataset('cifar10').get_dataloaders()
#     print(len(dataloaders['test']))







'''Configuration file for ColourGAN.'''

class ColourGANConfig:
    initial_weights_generator = None
    initial_weights_discriminator = None

    generator_normalization = 'batch'
    discriminator_normalization = 'batch'

    base_lr_generator = 3e-4
    base_lr_discriminator = 6e-5

    lr_decay = 0.1
    lr_decay_steps = 6e4

    l1_weight = 0.99

    data_path = ''
    output_path = 'output'
    training_stats_file = 'training_logs.txt'
    checkpoint_frequency = 5
    batch_size = 32
    num_workers = 4
    epochs = 100
    smoothing = 0.9

def get_cfg():
    return ColourGANConfig()


'''Interactive Testing Flask app for Model.'''
from colourgan.model import ColourGAN
from colourgan.config import get_cfg
import streamlit as st
from PIL import Image
import numpy as np

def app(weights):
    cfg = get_cfg()
    cfg.initial_weights_generator = weights
    model = ColourGAN(cfg, inference=True)

    st.title('ganapps ')
    st.write('\n')

    st.write("""
    ## ColourGAN Web Application
    """)
    uploaded_file = st.file_uploader("text here", type="png")
    if uploaded_file is not None:
        col1, col2 = st.beta_columns(2)
        image = Image.open(uploaded_file)
        col1.image(image, caption='Uploaded Image.',width = 32, use_column_width=True)
        new_img = model.inference(np.array(image))
        col2.image(new_img, caption='Coloured Image.', use_column_width=True)

    st.write("""
    [GitHub Repository](https://github.com/narenderkumarnain/ColourGAN)
    """)











'''Utility Functions for Model.'''
import os

def save_losses(cfg , epoch_gen_adv_loss, epoch_gen_l1_loss, epoch_disc_real_loss, epoch_disc_fake_loss,
                 epoch_disc_real_acc, epoch_disc_fake_acc, data_loader_len, l1_weight):
    """Create a display all the losses and accuracies."""
    if os.path.exists(os.path.join(cfg.output_path , cfg.training_stats_file)):
        with open(os.path.join(cfg.output_path , cfg.training_stats_file) , 'r') as f:
            data = f.read()
    else:
        data = ''

    log1 = '  Generator: adversarial loss = {:.4f}, L1 loss = {:.4f}, full loss = {:.4f}'.format(
        epoch_gen_adv_loss / data_loader_len,
        epoch_gen_l1_loss / data_loader_len,
        (epoch_gen_adv_loss / data_loader_len)*(1.0-l1_weight) + (epoch_gen_l1_loss / data_loader_len)*l1_weight
    )

    log2 = '  Discriminator: loss = {:.4f}'.format(
        (epoch_disc_real_loss + epoch_disc_fake_loss) / (data_loader_len*2)
    )

    log3 = '                 acc. = {:.4f} (real acc. = {:.4f}, fake acc. = {:.4f})'.format(
        (epoch_disc_real_acc + epoch_disc_fake_acc) / (data_loader_len*2),
        epoch_disc_real_acc / data_loader_len,
        epoch_disc_fake_acc / data_loader_len
    )
    with open(os.path.join(cfg.output_path , cfg.training_stats_file) , 'w') as f:
        f.write(data + '\n' + log1 + '\n' + log2 + '\n' + log3)




'''Model Class for ColourGAN.'''
import os
import torch
import numpy as np
import cv2
import torch.nn.functional as F
from colourgan.networks import (
    Generator ,
    Discriminator
)
from colourgan.config import get_cfg
from colourgan.data import Cifar10Dataset
from colourgan.utils import save_losses
from colourgan.logger import logger

class ColourGAN:
    '''Wrapper Class for ColourGAN.'''
    def __init__(self,config,inference=False):
        device = torch.device('cuda:0' if torch.cuda.is_available() else 'cpu')
        self.cfg = config
        self.generator = Generator(self.cfg.generator_normalization)
        if not inference:
            self.discriminator = Discriminator(self.cfg.discriminator_normalization)
        if self.cfg.initial_weights_generator is not None:
            self.generator.load_state_dict(torch.load(self.cfg.initial_weights_generator,map_location = torch.device(device)))
            logger.info(f'Generator Weights Loaded from {self.cfg.initial_weights_generator}')
        if self.cfg.initial_weights_discriminator is not None:
            if not inference:
                self.discriminator.load_state_dict(torch.load(self.cfg.initial_weights_discriminator,map_location = torch.device(device)))
                logger.info(f'Discriminator Weights Loaded from {self.cfg.initial_weights_discriminator}')


    def train(self,train_loader,test_loader,epochs=20,pretrained=None):
        device = torch.device('cuda:0' if torch.cuda.is_available() else 'cpu')
        # Logging of device
        self.generator.to(device)
        self.discriminator.to(device)

        data_loaders = {
            'train': train_loader ,
            'test': test_loader
        }
        optimizers = {
            'gen': torch.optim.Adam(self.generator.parameters(),
                                    lr=self.cfg.base_lr_generator,
                                    betas=(0.5, 0.999)),
            'disc': torch.optim.Adam(self.discriminator.parameters(),
                                     lr=self.cfg.base_lr_discriminator,
                                     betas=(0.5, 0.999))
        }

        losses = {
            'l1': torch.nn.L1Loss(reduction='mean'),
            'disc': torch.nn.BCELoss(reduction='mean')
        }

        if not os.path.exists(self.cfg.output_path):
            os.makedirs(self.cfg.output_path)

        if pretrained is not None:
            # we need to load the weight dictionaries
            self.generator.load_state_dict(torch.load(
                pretrained['gen'],
                map_location=device
            ))
            self.discriminator.load_state_dict(torch.load(
                pretrained['disc'],
                map_location=device
            ))

        global_step = 0
        for epoch in range(epochs):
            print(f'Epoch: {epoch}')
            for phase in ['train', 'test']:

                # running losses for generator
                epoch_gen_adv_loss = 0.0
                epoch_gen_l1_loss = 0.0

                # running losses for discriminator
                epoch_disc_real_loss = 0.0
                epoch_disc_fake_loss = 0.0
                epoch_disc_real_acc = 0.0
                epoch_disc_fake_acc = 0.0

                if phase == 'train':
                    print('TRAINING:')
                else:
                    print('VALIDATION:')

                for idx, sample in enumerate(data_loaders[phase]):

                    # get data
                    img_l, real_img_lab = sample[:, 0:1, :, :].float().to(device), sample.float().to(device)

                    # generate targets
                    target_ones = torch.ones(real_img_lab.size(0), 1).to(device)
                    target_zeros = torch.zeros(real_img_lab.size(0), 1).to(device)

                    if phase == 'train':
                        # adjust LR
                        global_step += 1
                        self.__adjust_lr(optimizers['gen'], global_step, base_lr=self.cfg.base_lr_generator,
                                             lr_decay_rate=self.cfg.lr_decay, lr_decay_steps=self.cfg.lr_decay_steps)
                        self.__adjust_lr(optimizers['disc'], global_step, base_lr=self.cfg.base_lr_discriminator,
                                             lr_decay_rate=self.cfg.lr_decay, lr_decay_steps=self.cfg.lr_decay_steps)

                        # reset generator gradients
                        optimizers['gen'].zero_grad()

                    # train / inference the generator
                    with torch.set_grad_enabled(phase == 'train'):
                        fake_img_ab = self.generator(img_l)
                        fake_img_lab = torch.cat([img_l, fake_img_ab], dim=1).to(device)

                        #print(fake_img_lab.shape)
                        # adv loss
                        adv_loss = losses['disc'](self.discriminator(fake_img_lab), target_ones)
                        # l1 loss
                        l1_loss = losses['l1'](real_img_lab[:, 1:, :, :], fake_img_ab)
                        # full gen loss
                        full_gen_loss = (1.0 - self.cfg.l1_weight) * adv_loss + (self.cfg.l1_weight * l1_loss)

                        if phase == 'train':
                            full_gen_loss.backward()
                            optimizers['gen'].step()

                    epoch_gen_adv_loss += adv_loss.item()
                    epoch_gen_l1_loss += l1_loss.item()

                    if phase == 'train':
                        # reset discriminator gradients
                        optimizers['disc'].zero_grad()

                    # train / inference the discriminator
                    with torch.set_grad_enabled(phase == 'train'):
                        prediction_real = self.discriminator(real_img_lab)
                        prediction_fake = self.discriminator(fake_img_lab.detach())

                        loss_real = losses['disc'](prediction_real, target_ones * self.cfg.smoothing)
                        loss_fake = losses['disc'](prediction_fake, target_zeros)
                        full_disc_loss = loss_real + loss_fake

                        if phase == 'train':
                            full_disc_loss.backward()
                            optimizers['disc'].step()

                    epoch_disc_real_loss += loss_real.item()
                    epoch_disc_fake_loss += loss_fake.item()
                    epoch_disc_real_acc += np.mean(prediction_real.detach().cpu().numpy() > 0.5)
                    epoch_disc_fake_acc += np.mean(prediction_fake.detach().cpu().numpy() <= 0.5)

                    # save the first sample for later
                    if phase == 'test' and idx == 0:
                        sample_real_img_lab = real_img_lab
                        sample_fake_img_lab = fake_img_lab

                # display losses
                save_losses(self.cfg , epoch_gen_adv_loss, epoch_gen_l1_loss,
                             epoch_disc_real_loss, epoch_disc_fake_loss,
                             epoch_disc_real_acc, epoch_disc_fake_acc,
                             len(data_loaders[phase]), self.cfg.l1_weight)

                # save after every nth epoch
                if phase == 'test':
                    if epoch % self.cfg.checkpoint_frequency == 0 or epoch == self.cfg.max_epoch - 1:
                        gen_path = os.path.join(self.cfg.output_path, 'checkpoint_ep{}_gen.pt'.format(epoch))
                        disc_path = os.path.join(self.cfg.output_path, 'checkpoint_ep{}_disc.pt'.format(epoch))
                        torch.save(self.generator.state_dict(), gen_path)
                        torch.save(self.discriminator.state_dict(), disc_path)
                        print('Checkpoint.')


    def inference(self,img):
        img_bgr = img.astype(np.float32) / 255.0
        # transform to LAB
        img_lab = cv2.cvtColor(img_bgr, cv2.COLOR_BGR2LAB)
        img_lab[:, :, 0] = img_lab[:, :, 0] / 50 - 1
        img_lab[:, :, 1] = img_lab[:, :, 1] / 127
        img_lab[:, :, 2] = img_lab[:, :, 2] / 127
        img_lab = img_lab.transpose((2, 0, 1))
        out = self.generator(torch.tensor([img_lab[0:1,:,:]]))
        print(img_lab.shape)
        img_temp = F.interpolate(torch.tensor([img_lab[0:1,:,:]]), size=(32,32),mode='bilinear',align_corners=True)
        print(img_temp.shape , out.shape)
        out = torch.cat([img_temp , out], dim=1).detach().cpu().numpy()
        out = out[0].transpose((1, 2, 0))
        out[:, :, 0] = (out[:, :, 0] + 1) * 50
        out[:, :, 1] = out[:, :, 1] * 127
        out[:, :, 2] = out[:, :, 2] * 127
        # transform to bgr
        out = cv2.cvtColor(out, cv2.COLOR_LAB2BGR)
        # to int8
        out = (out * 255.0).astype(np.uint8)

        return out

    def __adjust_lr(self,optimizer, global_step, base_lr, lr_decay_rate=0.1, lr_decay_steps=6e4):
        """Adjust the learning rate of the params of an optimizer."""
        lr = base_lr * (lr_decay_rate ** (global_step/lr_decay_steps))
        if lr < 1e-6:
            lr = 1e-6

        for param_group in optimizer.param_groups:
            param_group['lr'] = lr


# if __name__ == '__main__':
#     import cv2
#     # dataloaders = Cifar10Dataset('cifar10').get_dataloaders()
#     cfg = get_cfg()
#     cfg.initial_weights_generator = 'output/checkpoint_ep99_gen.pt'
#     model = ColourGAN(cfg,inference=True)
#     # model.train(dataloaders['train'],dataloaders['test'])
#     img = cv2.imread('output/airbus_s_000413.png')
#     out = model.inference(img)
#     cv2.imwrite('output/temp2.png', out)




'''Logger for Complete Project.'''
import logging
import os

logging.basicConfig(
    format='%(asctime)s - ColourGAN - %(message)s',
    datefmt='%H:%M:%S',
    level=logging.INFO)
logger = logging.getLogger(__name__)

# if not os.path.exists('logs'):
#     os.mkdir('logs')
#
# file = open(os.path.join('logs','log_history.txt') , 'w')

def log(message, info = 'ColourGAN'):
    # Something Else
    logger.info(message)
    # file.write(info + ' : ' +  message + '\n')
    return



import setuptools

with open("README.md", "r", encoding="utf-8") as fh:
    long_description = fh.read()

setuptools.setup(
    name="colourgan",
    version="1.0.3",
    author="Narender Nain",
    author_email="narenderkumarnain@gmail.com",
    description="GAN Based colouring model",
    long_description=long_description,
    long_description_content_type="text/markdown",
    url="https://github.com/narenderkumarnain/GAN-Apps",
    project_urls={
    },
    install_requires=['numpy','opencv-python==4.5.2.52' ,
                      'Pillow==8.2.0', 'torch==1.8.1','torchvision==0.9.1'],
    classifiers=[
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
    ],
    package_dir={"": "."},
    packages=setuptools.find_packages(),
    python_requires=">=3.6",
)